@@ -1,11 +1,11 @@
 /*
  * SPDX-License-Identifier: GPL-3.0-only
- * MuseScore-CLA-applies
+ * MuseScore-Studio-CLA-applies
  *
- * MuseScore
+ * MuseScore Studio
  * Music Composition & Notation
  *
- * Copyright (C) 2021 MuseScore BVBA and others
+ * Copyright (C) 2021 MuseScore Limited
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -72,8 +72,13 @@ Ambitus::Ambitus(const Ambitus& a)
     m_direction = a.m_direction;
     m_hasLine = a.m_hasLine;
     m_lineWidth = a.m_lineWidth;
+
     m_topAccidental = a.m_topAccidental->clone();
+    m_topAccidental->setParent(this);
+
     m_bottomAccidental = a.m_bottomAccidental->clone();
+    m_bottomAccidental->setParent(this);
+
     m_topPitch = a.m_topPitch;
     m_topTpc = a.m_topTpc;
     m_bottomPitch = a.m_bottomPitch;
@@ -301,7 +306,7 @@ double Ambitus::headWidth() const
 //   pagePos
 //---------------------------------------------------------
 
-mu::PointF Ambitus::pagePos() const
+PointF Ambitus::pagePos() const
 {
     if (explicitParent() == 0) {
         return pos();
@@ -424,7 +429,7 @@ PropertyValue Ambitus::getProperty(Pid propertyId) const
         return int(direction());
     case Pid::GHOST:                         // recycled property = _hasLine
         return hasLine();
-    case Pid::LINE_WIDTH_SPATIUM:
+    case Pid::LINE_WIDTH:
         return lineWidth();
     case Pid::TPC1:
         return topTpc();
@@ -462,7 +467,7 @@ bool Ambitus::setProperty(Pid propertyId, const PropertyValue& v)
     case Pid::GHOST:                         // recycled property = _hasLine
         setHasLine(v.toBool());
         break;
-    case Pid::LINE_WIDTH_SPATIUM:
+    case Pid::LINE_WIDTH:
         setLineWidth(v.value<Spatium>());
         break;
     case Pid::TPC1:
@@ -505,7 +510,7 @@ PropertyValue Ambitus::propertyDefault(Pid id) const
         return int(DIRECTION_DEFAULT);
     case Pid::GHOST:
         return HASLINE_DEFAULT;
-    case Pid::LINE_WIDTH_SPATIUM:
+    case Pid::LINE_WIDTH:
         return Spatium(LINEWIDTH_DEFAULT);
     case Pid::TPC1:
         return estimateRanges().topTpc;
@@ -549,8 +554,11 @@ EngravingItem* Ambitus::prevSegmentElement()
 
 String Ambitus::accessibleInfo() const
 {
+    if (m_topTpc == Tpc::TPC_INVALID || m_bottomTpc == Tpc::TPC_INVALID) {
+        return EngravingItem::accessibleInfo();
+    }
     return EngravingItem::accessibleInfo() + u"; "
-           + mtrc("engraving", "Top pitch: %1; Bottom pitch: %2")
+           + muse::mtrc("engraving", "Top pitch: %1; Bottom pitch: %2")
            .arg(tpc2name(topTpc(), NoteSpellingType::STANDARD, NoteCaseType::AUTO, false) + String::number(topOctave()),
                 tpc2name(bottomTpc(), NoteSpellingType::STANDARD, NoteCaseType::AUTO, false) + String::number(bottomOctave()));
 }
