@@ -1,11 +1,11 @@
 /*
  * SPDX-License-Identifier: GPL-3.0-only
- * MuseScore-CLA-applies
+ * MuseScore-Studio-CLA-applies
  *
- * MuseScore
+ * MuseScore Studio
  * Music Composition & Notation
  *
- * Copyright (C) 2021 MuseScore BVBA and others
+ * Copyright (C) 2021 MuseScore Limited
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -24,7 +24,6 @@
 #include "io/buffer.h"
 
 #include "compat/writescorehook.h"
-#include "infrastructure/mscwriter.h"
 
 #include "rw/mscloader.h"
 #include "rw/xmlreader.h"
@@ -34,10 +33,10 @@
 
 #include "engravingproject.h"
 
-#include "audio.h"
 #include "barline.h"
 #include "excerpt.h"
 #include "factory.h"
+#include "box.h"
 #include "linkedobjects.h"
 #include "repeatlist.h"
 #include "rest.h"
@@ -49,15 +48,15 @@
 #include "log.h"
 
 using namespace mu;
-using namespace mu::io;
+using namespace muse::io;
 using namespace mu::engraving;
 
 //---------------------------------------------------------
 //   MasterScore
 //---------------------------------------------------------
 
-MasterScore::MasterScore(std::weak_ptr<engraving::EngravingProject> project)
-    : Score()
+MasterScore::MasterScore(const muse::modularity::ContextPtr& iocCtx, std::weak_ptr<engraving::EngravingProject> project)
+    : Score(iocCtx)
 {
     m_project = project;
     m_undoStack   = new UndoStack();
@@ -67,10 +66,6 @@ MasterScore::MasterScore(std::weak_ptr<engraving::EngravingProject> project)
     m_nonExpandedRepeatList = new RepeatList(this);
     setMasterScore(this);
 
-    m_pos[int(POS::CURRENT)] = Fraction(0, 1);
-    m_pos[int(POS::LEFT)]    = Fraction(0, 1);
-    m_pos[int(POS::RIGHT)]   = Fraction(0, 1);
-
 #if defined(Q_OS_WIN)
     metaTags().insert({ u"platform", u"Microsoft Windows" });
 #elif defined(Q_OS_MAC)
@@ -90,11 +85,11 @@ MasterScore::MasterScore(std::weak_ptr<engraving::EngravingProject> project)
     metaTags().insert({ u"translator", u"" });
     metaTags().insert({ u"source", u"" });
     metaTags().insert({ u"copyright", u"" });
-    metaTags().insert({ u"creationDate", Date::currentDate().toString(DateFormat::ISODate) });
+    metaTags().insert({ u"creationDate", muse::Date::currentDate().toString(muse::DateFormat::ISODate) });
 }
 
-MasterScore::MasterScore(const MStyle& s, std::weak_ptr<engraving::EngravingProject> project)
-    : MasterScore{project}
+MasterScore::MasterScore(const muse::modularity::ContextPtr& iocCtx, const MStyle& s, std::weak_ptr<engraving::EngravingProject> project)
+    : MasterScore{iocCtx, project}
 {
     setStyle(s);
 }
@@ -110,7 +105,7 @@ MasterScore::~MasterScore()
     delete m_sigmap;
     delete m_tempomap;
     delete m_undoStack;
-    DeleteAll(m_excerpts);
+    muse::DeleteAll(m_excerpts);
 }
 
 //---------------------------------------------------------
@@ -208,14 +203,14 @@ const RepeatList& MasterScore::repeatList() const
     return *m_nonExpandedRepeatList;
 }
 
-const RepeatList& MasterScore::repeatList(bool expandRepeats) const
+const RepeatList& MasterScore::repeatList(bool expandRepeats, bool updateTies) const
 {
     if (expandRepeats) {
-        m_expandedRepeatList->update(true);
+        m_expandedRepeatList->update(true, updateTies);
         return *m_expandedRepeatList;
     }
 
-    m_nonExpandedRepeatList->update(false);
+    m_nonExpandedRepeatList->update(false, updateTies);
     return *m_nonExpandedRepeatList;
 }
 
@@ -229,7 +224,7 @@ void MasterScore::addExcerpt(Excerpt* ex, size_t index)
         initParts(ex);
     }
 
-    excerpts().insert(excerpts().begin() + (index == mu::nidx ? excerpts().size() : index), ex);
+    excerpts().insert(excerpts().begin() + (index == muse::nidx ? excerpts().size() : index), ex);
     setExcerptsChanged(true);
 }
 
@@ -239,7 +234,7 @@ void MasterScore::addExcerpt(Excerpt* ex, size_t index)
 
 void MasterScore::removeExcerpt(Excerpt* ex)
 {
-    if (mu::remove(excerpts(), ex)) {
+    if (muse::remove(excerpts(), ex)) {
         setExcerptsChanged(true);
         // delete ex;
     } else {
@@ -256,17 +251,16 @@ MasterScore* MasterScore::clone()
     Buffer buffer;
     buffer.open(IODevice::WriteOnly);
 
-    rw::RWRegister::writer()->writeScore(this, &buffer, false);
+    rw::RWRegister::writer(iocContext())->writeScore(this, &buffer);
 
     buffer.close();
 
-    ByteArray scoreData = buffer.data();
-    MasterScore* score = new MasterScore(style(), m_project);
+    muse::ByteArray scoreData = buffer.data();
+    MasterScore* score = new MasterScore(iocContext(), style(), m_project);
 
     XmlReader r(scoreData);
     MscLoader().readMasterScore(score, r, true);
 
-    score->addLayoutFlags(LayoutFlag::FIX_PITCH_VELO);
     score->doLayout();
     return score;
 }
@@ -285,8 +279,19 @@ Score* MasterScore::createScore(const MStyle& s)
 //   setPos
 //---------------------------------------------------------
 
-void MasterScore::setPos(POS pos, Fraction tick)
+Fraction MasterScore::loopBoundaryTick(LoopBoundaryType type) const
 {
+    IF_ASSERT_FAILED(type != LoopBoundaryType::Unknown) {
+        return Fraction();
+    }
+    return m_loopBoundaries[size_t(type)];
+}
+
+void MasterScore::setLoopBoundaryTick(LoopBoundaryType type, Fraction tick)
+{
+    IF_ASSERT_FAILED(type != LoopBoundaryType::Unknown) {
+        return;
+    }
     if (tick < Fraction(0, 1)) {
         tick = Fraction(0, 1);
     }
@@ -295,12 +300,12 @@ void MasterScore::setPos(POS pos, Fraction tick)
         tick = lastMeasure()->endTick();
     }
 
-    m_pos[int(pos)] = tick;
+    m_loopBoundaries[size_t(type)] = tick;
     // even though tick position might not have changed, layout might have
     // so we should update cursor here
-    // however, we must be careful not to call setPos() again while handling posChanged, or recursion results
+    // however, we must be careful not to call setLoopBoundaryTick() again while handling posChanged, or recursion results
     for (Score* s : scoreList()) {
-        s->notifyPosChanged(pos, unsigned(tick.ticks()));
+        s->notifyLoopBoundaryTickChanged(type, unsigned(tick.ticks()));
     }
 }
 
@@ -324,8 +329,8 @@ void MasterScore::setLayoutAll(staff_idx_t staff, const EngravingItem* e)
 
     if (e && e->score() == this) {
         // TODO: map staff number properly
-        const staff_idx_t startStaff = staff == mu::nidx ? 0 : staff;
-        const staff_idx_t endStaff = staff == mu::nidx ? (nstaves() - 1) : staff;
+        const staff_idx_t startStaff = staff == muse::nidx ? 0 : staff;
+        const staff_idx_t endStaff = staff == muse::nidx ? (nstaves() - 1) : staff;
         m_cmdState.setStaff(startStaff);
         m_cmdState.setStaff(endStaff);
 
@@ -547,7 +552,7 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
         newMeasure->setTimesig(currentTimeSig);
         newMeasure->setTicks(currentTimeSig);
 
-        undo(new InsertMeasures(newMeasure, newMeasure));
+        undo(new InsertMeasures(newMeasure, newMeasure, options.moveStaffTypeChanges));
 
         ticks = newMeasure->ticks();
         Measure* measureInsert = nullptr; // insert before
@@ -565,20 +570,22 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
 
         std::vector<TimeSig*> timeSigList;
         std::vector<KeySig*> keySigList;
-        std::vector<Clef*> clefList;
+        std::vector<Clef*> initClefList;
         std::vector<Clef*> previousClefList;
         std::list<Clef*> specialCaseClefs;
+        std::vector<Clef*> afterBarlineClefs;
         std::vector<BarLine*> previousBarLinesList;
 
         Measure* pm = newMeasure->prevMeasure();
 
         //
-        // remove clef, barlines, time and key signatures
+        // remove clefs, barlines, time and key signatures
         //
         bool headerKeySig = false;
         if (measureInsert) {
             // if inserting before first measure, always preserve clefs and signatures
             // at the begining of the score (move them back)
+
             if (pm && !options.moveSignaturesClef && !isBeginning) {
                 Segment* ps = pm->findSegment(SegmentType::Clef, tick);
                 if (ps && ps->enabled()) {
@@ -602,18 +609,32 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
                             continue;
                         }
                         EngravingItem* e = s->element(staffIdx * VOICES);
-                        // if it's a clef, we only add if it's a header clef
-                        bool moveClef = s->isHeaderClefType() && e && e->isClef() && !toClef(e)->forInstrumentChange();
-                        // otherwise, we only add if e exists and is generated
-                        bool moveOther = e && !e->isClef() && (!e->generated() || tick.isZero());
                         bool specialCase = false;
-                        if (e && e->isClef() && !moveClef && isBeginning
-                            && toClef(e)->clefToBarlinePosition() != ClefToBarlinePosition::AFTER) {
-                            // special case:
-                            // there is a non-header clef at global tick 0, and we are inserting at the beginning of the score.
-                            // this clef will be moved with the measure it accompanies, but it will be moved before the barline.
-                            specialCase = true;
-                            moveClef = true;
+                        bool initClef = false;
+                        bool moveClef = false;
+                        bool moveOther = false;
+                        if (e && e->isClef()) {
+                            if (s->isHeaderClefType()) {
+                                // if it's a header clef, we only add if it's an init clef
+                                // (other header clefs are handled by undo(new InsertMeasures())
+                                initClef = isBeginning && !toClef(e)->forInstrumentChange();
+                                moveClef = initClef;
+                            } else {
+                                ClefToBarlinePosition clefPos = toClef(e)->clefToBarlinePosition();
+                                if (isBeginning) {
+                                    // special case:
+                                    // there is a non-header clef at global tick 0, and we are inserting at the beginning of the score.
+                                    // this clef will be moved with the measure it accompanies, but it will be moved before the barline.
+                                    specialCase = true;
+                                    moveClef = true;
+                                } else if (clefPos == ClefToBarlinePosition::AFTER) {
+                                    // non header clef at the begining of the measure
+                                    moveClef = true;
+                                }
+                            }
+                        } else {
+                            // otherwise, we only add if e is not generated
+                            moveOther = e && (!e->generated() || tick.isZero());
                         }
                         if (!moveClef && !moveOther) {
                             continue; // this item will remain with the old measure
@@ -644,23 +665,21 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
                             TimeSig* ts = toTimeSig(e);
                             timeSigList.push_back(ts);
                             ee = e;
-                        }
-                        if (specialCase) {
-                            specialCaseClefs.push_back(toClef(e));
-                            ee = e;
-                        } else if (tick.isZero() && e->isClef()) {
+                        } else if (e->isClef()) {
                             Clef* clef = toClef(e);
-                            clefList.push_back(clef);
+                            if (specialCase) {
+                                specialCaseClefs.push_back(clef);
+                            } else if (initClef) {
+                                initClefList.push_back(clef);
+                            } else {
+                                afterBarlineClefs.push_back(toClef(e));
+                            }
                             ee = e;
                         }
                         if (ee) {
                             doUndoRemoveElement(ee);
                             if (s->empty() && s->isTimeSigType()) {
-                                if (ee->isTimeSig()) {
-                                    undoRemoveElement(s);
-                                } else {
-                                    s->setEnabled(false);
-                                }
+                                undoRemoveElement(s);
                             }
                         }
                     }
@@ -677,7 +696,7 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
             score->restoreInitialKeySigAndTimeSig();
         }
 
-        if (pm && !options.moveSignaturesClef) {
+        if (pm && !options.moveSignaturesClef && !options.ignoreBarLines) {
             Segment* pbs = pm->findSegment(SegmentType::EndBarLine, tick);
             if (pbs && pbs->enabled()) {
                 for (size_t staffIdx = 0; staffIdx < score->nstaves(); ++staffIdx) {
@@ -704,7 +723,7 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
             TimeSig* nts = Factory::copyTimeSig(*ts);
             Segment* s   = newMeasure->undoGetSegmentR(SegmentType::TimeSig, Fraction(0, 1));
             nts->setParent(s);
-            undoAddElement(nts);
+            doUndoAddElement(nts);
         }
         for (KeySig* ks : keySigList) {
             KeySig* nks = Factory::copyKeySig(*ks);
@@ -716,32 +735,39 @@ MeasureBase* MasterScore::insertMeasure(MeasureBase* beforeMeasure, const Insert
             if (!nks->isAtonal()) {
                 nks->setKey(nks->concertKey());  // to set correct (transposing) key
             }
-            undoAddElement(nks);
+            doUndoAddElement(nks);
         }
-        for (Clef* clef : clefList) {
+        for (Clef* clef : initClefList) {
             Clef* nClef = Factory::copyClef(*clef);
             Segment* s  = newMeasure->undoGetSegmentR(SegmentType::HeaderClef, Fraction(0, 1));
             s->setHeader(true);
             nClef->setParent(s);
-            undoAddElement(nClef);
+            doUndoAddElement(nClef);
+        }
+        for (Clef* clef : afterBarlineClefs) {
+            Clef* nClef = Factory::copyClef(*clef);
+            Segment* s  = newMeasure->undoGetSegmentR(SegmentType::Clef, Fraction(0, 1));
+            s->setHeader(true);
+            nClef->setParent(s);
+            doUndoAddElement(nClef);
         }
         for (Clef* clef : previousClefList) {
             Clef* nClef = Factory::copyClef(*clef);
             Segment* s  = newMeasure->undoGetSegmentR(SegmentType::Clef, newMeasure->ticks());
             nClef->setParent(s);
-            undoAddElement(nClef);
+            doUndoAddElement(nClef);
         }
         for (Clef* clef : specialCaseClefs) {
             Clef* nClef = Factory::copyClef(*clef);
             Segment* s  = newMeasure->undoGetSegmentR(SegmentType::Clef, newMeasure->ticks());
             nClef->setParent(s);
-            undoAddElement(nClef);
+            doUndoAddElement(nClef);
         }
         for (BarLine* barLine : previousBarLinesList) {
             BarLine* nBarLine = Factory::copyBarLine(*barLine);
             Segment* s = newMeasure->undoGetSegmentR(SegmentType::EndBarLine, newMeasure->ticks());
             nBarLine->setParent(s);
-            undoAddElement(nBarLine);
+            doUndoAddElement(nBarLine);
         }
     }
 
