@@ -25,7 +25,7 @@
 #include <cstring>
 #include <zlib.h>
 
-#include "io/dir.h"
+#include "global/io/dir.h"
 
 #include "log.h"
 
@@ -39,13 +39,13 @@
 #define ZDEBUG if (0) LOGD
 #endif
 
-using namespace mu::io;
+using namespace muse::io;
 
 typedef unsigned long int ulong;
 typedef unsigned short int ushort;
 typedef unsigned int uint;
 
-namespace mu {
+namespace muse {
 static inline uint readUInt(const uint8_t* data)
 {
     return (data[0]) + (data[1] << 8) + (data[2] << 16) + (data[3] << 24);
@@ -393,6 +393,8 @@ struct ZipContainer::Impl {
 
     void scanFiles();
     ZipContainer::FileInfo fillFileInfo(size_t index) const;
+
+    std::string fixFilePath(const ByteArray& path) const;
 };
 
 void ZipContainer::Impl::scanFiles()
@@ -531,31 +533,11 @@ ZipContainer::FileInfo ZipContainer::Impl::fillFileInfo(size_t index) const
     // ushort general_purpose_bits = readUShort(header.h.general_purpose_bits);
     // if bit 11 is set, the filename and comment fields must be encoded using UTF-8
     // const bool inUtf8 = (general_purpose_bits & Utf8Names) != 0;
-    fileInfo.filePath = header.file_name.constChar();
+    fileInfo.filePath = fixFilePath(header.file_name);
     fileInfo.crc = readUInt(header.h.crc_32);
     fileInfo.size = readUInt(header.h.uncompressed_size);
     fileInfo.lastModified = readMSDosDate(header.h.last_mod_file);
 
-    // fix the file path, if broken (convert separators, eat leading and trailing ones)
-    fileInfo.filePath = Dir::fromNativeSeparators(fileInfo.filePath).toStdString();
-    {
-        bool frontOk = false;
-        while (!fileInfo.filePath.empty() && !frontOk) {
-            if (fileInfo.filePath.front() == '/') {
-                fileInfo.filePath = fileInfo.filePath.substr(1);
-            } else if (fileInfo.filePath.rfind("./", 0) == 0) {
-                fileInfo.filePath = fileInfo.filePath.substr(2);
-            } else if (fileInfo.filePath.rfind("../", 0) == 0) {
-                fileInfo.filePath = fileInfo.filePath.substr(3);
-            } else {
-                frontOk = true;
-            }
-        }
-    }
-    while (!fileInfo.filePath.empty() && fileInfo.filePath.back() == '/') {
-        fileInfo.filePath = fileInfo.filePath.substr(0, fileInfo.filePath.size() - 1);
-    }
-
     return fileInfo;
 }
 
@@ -585,8 +567,13 @@ void ZipContainer::Impl::addEntry(EntryType type, const std::string& fileName, c
     writeUInt(header.h.uncompressed_size, (uint)contents.size());
 
     std::time_t t = std::time(0);   // get time now
-    std::tm* now = std::localtime(&t);
-    writeMSDosDate(header.h.last_mod_file, *now);
+    std::tm now;
+#ifdef WIN32
+    localtime_s(&now, &t);
+#else
+    localtime_r(&t, &now);
+#endif
+    writeMSDosDate(header.h.last_mod_file, now);
     ByteArray data = contents;
     if (compression == ZipContainer::AlwaysCompress) {
         writeUShort(header.h.compression_method, CompressionMethodDeflated);
@@ -688,6 +675,32 @@ bool ZipContainer::Impl::writeToDevice(const ByteArray& data)
     return device->write(data) == data.size();
 }
 
+std::string ZipContainer::Impl::fixFilePath(const ByteArray& path) const
+{
+    // fix the file path, if broken (convert separators, eat leading and trailing ones)
+    std::string fixed = Dir::fromNativeSeparators(path.constChar()).toStdString();
+    {
+        bool frontOk = false;
+        while (!fixed.empty() && !frontOk) {
+            if (fixed.front() == '/') {
+                fixed = fixed.substr(1);
+            } else if (fixed.rfind("./", 0) == 0) {
+                fixed = fixed.substr(2);
+            } else if (fixed.rfind("../", 0) == 0) {
+                fixed = fixed.substr(3);
+            } else {
+                frontOk = true;
+            }
+        }
+    }
+
+    while (!fixed.empty() && fixed.back() == '/') {
+        fixed = fixed.substr(0, fixed.size() - 1);
+    }
+
+    return fixed;
+}
+
 ZipContainer::ZipContainer(IODevice* device)
     : p(new Impl(device))
 {
@@ -721,12 +734,14 @@ int ZipContainer::count() const
 bool ZipContainer::fileExists(const std::string& fileName) const
 {
     p->scanFiles();
-    ByteArray fileNameBa = ByteArray::fromRawData(fileName.c_str(), fileName.size());
+
     for (size_t i = 0; i < p->fileHeaders.size(); ++i) {
-        if (p->fileHeaders.at(i).file_name == fileNameBa) {
+        std::string currentFileName = p->fixFilePath(p->fileHeaders.at(i).file_name.constChar());
+        if (currentFileName == fileName) {
             return true;
         }
     }
+
     return false;
 }
 
@@ -734,11 +749,10 @@ ByteArray ZipContainer::fileData(const std::string& fileName) const
 {
     p->scanFiles();
 
-    ByteArray fileNameBa = ByteArray::fromRawData(fileName.c_str(), fileName.size());
-
-    size_t i;
+    size_t i = 0;
     for (i = 0; i < p->fileHeaders.size(); ++i) {
-        if (p->fileHeaders.at(i).file_name == fileNameBa) {
+        std::string currentFileName = p->fixFilePath(p->fileHeaders.at(i).file_name.constChar());
+        if (currentFileName == fileName) {
             break;
         }
     }
